#include "rastmath.h"
#include "triangle.h"
#include <memory.h>

////////////////////////////////////////////////////
///////////////////   Vector   /////////////////////
////////////////////////////////////////////////////

void Vector::transform(const Matrix4x4 &m)
{
    float newx, newy;
    newx = x*m.m[0][0] + y*m.m[0][1] + z*m.m[0][2] + m.m[0][3];
    newy = x*m.m[1][0] + y*m.m[1][1] + z*m.m[1][2] + m.m[1][3];
    z = x*m.m[2][0] + y*m.m[2][1] + z*m.m[2][2] + m.m[2][3];
    x = newx;
    y = newy;
};

void Vector::Normalize()
{
	float length;

	length = (float)sqrt(x*x + y*y + z*z);
	if (length)  {
		x /=length;
		y /=length;
		z /=length;
	}
}

////////////////////////////////////////////////////
//////////////////   Matrix   //////////////////////
////////////////////////////////////////////////////

/////////  the mighty constructors!!!   ////////////

Matrix4x4::Matrix4x4()
{
	memset(this, 0, sizeof(Matrix4x4));
	m[0][0] = m[1][1] = m[2][2] = m[3][3] = 1;
}

Matrix4x4::Matrix4x4(Matrix4x4 &mat)
{
	memcpy(this, &mat, sizeof(Matrix4x4));
}

/////////////////////////////////////////////////////


Matrix4x4 Matrix4x4::operator +(const Matrix4x4 &mat) const
{
	Matrix4x4 mf;
	int i, j;

	for(i=0; i<4; i++) 
		for(j=0; j<4; j++) 
			mf.m[i][j] = m[i][j] + mat.m[i][j];

	return mf;
}

Matrix4x4 Matrix4x4::operator -(const Matrix4x4 &mat) const
{
	Matrix4x4 mf;
	int i, j;

	for(i=0; i<4; i++) 
		for(j=0; j<4; j++) 
			mf.m[i][j] = m[i][j] - mat.m[i][j];

	return mf;
}
	
	
	
Matrix4x4 Matrix4x4::operator *(const Matrix4x4 &mat) const
{
	Matrix4x4 mf;
	int i, j;

	for(i=0; i<4; i++)  
		for(j=0; j<4; j++)  
			mf.m[j][i] = m[i][0]*mat.m[0][j] + m[i][1]*mat.m[1][j] + m[i][2]*mat.m[2][j] + m[i][3]*mat.m[3][j];		

	return mf;
}


void Matrix4x4::operator +=(const Matrix4x4 &mat)
{
	int i, j;

	for(i=0; i<4; i++) 
		for(j=0; j<4; j++) 
			m[i][j] += mat.m[i][j];
}


void Matrix4x4::operator -=(const Matrix4x4 &mat)
{
	int i, j;

	for(i=0; i<4; i++) 
		for(j=0; j<4; j++) 
			m[i][j] -= mat.m[i][j];
}


///////////////////////////////////////////////////////////

void Matrix4x4::reset()
{
	memset(this, 0, sizeof(Matrix4x4));
	m[0][0] = m[1][1] = m[2][2] = m[3][3] = 1;
}

///////////////////////////////////////////////////////////

void Matrix4x4::Translate(float x, float y, float z)
{
	Matrix4x4 mat;
	
	mat.m[0][3] = x;
	mat.m[1][3] = y;
	mat.m[2][3] = z;

	(*this) *= mat;
}


void Matrix4x4::Rotate(float x, float y, float z)
{
	Matrix4x4 rotx, roty, rotz;

	rotx.m[1][1] = rotx.m[2][2] = float(cos(x));
	rotx.m[1][2] = -float(sin(x));
	rotx.m[2][1] = float(sin(x));

	roty.m[0][0] = roty.m[2][2] = float(cos(y));
	roty.m[2][0] = -float(sin(y));
	roty.m[0][2] = float(sin(y));

	rotz.m[0][0] = rotz.m[1][1] = float(cos(z));
	rotz.m[0][1] = -float(sin(z));
	rotz.m[1][0] = float(sin(z));

	(*this) *= rotx;
	(*this) *= roty;
	(*this) *= rotz;
}



void Matrix4x4::Scale(float x, float y, float z)
{
	Matrix4x4 mat;

	mat.m[0][0] = x;
	mat.m[1][1] = y;
	mat.m[2][2] = z;

	(*this) *= mat;
}

///////////////////////////////////////////////////////

void Triangle::CalculateNormal()
{
	Vector vec1, vec2, vec3;

	vec1 = vert[1]->pos - vert[0]->pos;
	vec2 = vert[2]->pos - vert[0]->pos;

	norm = vec1.crossproduct(vec2);
	norm.Normalize();
}

